// client.cs
//
// This program allows an arbitrary # of connections to be made to the associated server program
// in order to test the connection limits of a given system.  It also demonstrates how to use
// completely asynchronous I/O operations on sockets for both connection management as well
// as I/O.
//
// Refer to server.cs for the server implementation and utils.cs for the shared
// ThreadSafeControlUpdater utility class.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;

namespace Client
{
    // ServerInfo
    //
    // One instance of this class is allocated for each connection
    // the client makes to the server.
    //
    class ServerInfo
    {
        internal ServerInfo( Socket s )
        {
            socket = s;
        }

        internal readonly Socket socket;
        internal byte[] replyBuf = new byte[32];
        internal bool shutdown;
    }

	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
        delegate void UpdateConnectionCountDelegate( int howMany );

        private ArrayList connectionList = ArrayList.Synchronized(new ArrayList());
        private ServerInfo lastServer;
        private AsyncCallback connectedProc;
        private AsyncCallback receiveProc;
        private byte[] scratchBuffer = new byte[32];

        // Thread-affine control updaters for controls being updated
        // from I/O threads.
        //
        private ThreadSafeControlUpdater lastEndpointUpdater;
        private ThreadSafeControlUpdater replyTextUpdater;
        private ThreadSafeControlUpdater lastExceptionUpdater;
        private UpdateConnectionCountDelegate updateConnectionCountProc;

        private System.Windows.Forms.TextBox portNum;
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.TextBox hostName;
        private System.Windows.Forms.Label label4;
        private System.Windows.Forms.TextBox numConnections;
        private System.Windows.Forms.GroupBox groupBox1;
        private System.Windows.Forms.GroupBox groupBox2;
        private System.Windows.Forms.GroupBox groupBox3;
        private System.Windows.Forms.Button disconnect;
        private System.Windows.Forms.Label connectionCount;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Button connect;
        private System.Windows.Forms.Label label6;
        private System.Windows.Forms.TextBox textToSend;
        private System.Windows.Forms.Button sendText;
        private System.Windows.Forms.Label label7;
        private System.Windows.Forms.Label replyText;
        private System.Windows.Forms.TextBox lastException;
        private System.Windows.Forms.Button clearException;
        private System.Windows.Forms.Label label5;
        private System.Windows.Forms.Label lastEndpoint;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

            connectedProc = new AsyncCallback(OnConnect);
            receiveProc = new AsyncCallback(OnReceive);

            // Initialize the thread-affine control updaters.
            //
            lastEndpointUpdater = new ThreadSafeControlUpdater(lastEndpoint);
            replyTextUpdater = new ThreadSafeControlUpdater(replyText);
            lastExceptionUpdater = new ThreadSafeControlUpdater(lastException);
            updateConnectionCountProc = new UpdateConnectionCountDelegate(UpdateConnectionCount);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.portNum = new System.Windows.Forms.TextBox();
            this.label1 = new System.Windows.Forms.Label();
            this.label3 = new System.Windows.Forms.Label();
            this.hostName = new System.Windows.Forms.TextBox();
            this.label4 = new System.Windows.Forms.Label();
            this.numConnections = new System.Windows.Forms.TextBox();
            this.groupBox1 = new System.Windows.Forms.GroupBox();
            this.groupBox2 = new System.Windows.Forms.GroupBox();
            this.groupBox3 = new System.Windows.Forms.GroupBox();
            this.disconnect = new System.Windows.Forms.Button();
            this.connectionCount = new System.Windows.Forms.Label();
            this.label2 = new System.Windows.Forms.Label();
            this.connect = new System.Windows.Forms.Button();
            this.label6 = new System.Windows.Forms.Label();
            this.textToSend = new System.Windows.Forms.TextBox();
            this.sendText = new System.Windows.Forms.Button();
            this.label7 = new System.Windows.Forms.Label();
            this.replyText = new System.Windows.Forms.Label();
            this.lastException = new System.Windows.Forms.TextBox();
            this.clearException = new System.Windows.Forms.Button();
            this.label5 = new System.Windows.Forms.Label();
            this.lastEndpoint = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // portNum
            // 
            this.portNum.Location = new System.Drawing.Point(200, 32);
            this.portNum.Name = "portNum";
            this.portNum.Size = new System.Drawing.Size(56, 20);
            this.portNum.TabIndex = 4;
            this.portNum.Text = "999";
            // 
            // label1
            // 
            this.label1.Location = new System.Drawing.Point(168, 32);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(32, 23);
            this.label1.TabIndex = 3;
            this.label1.Text = "Port:";
            // 
            // label3
            // 
            this.label3.Location = new System.Drawing.Point(16, 32);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(40, 23);
            this.label3.TabIndex = 1;
            this.label3.Text = "Host:";
            // 
            // hostName
            // 
            this.hostName.Location = new System.Drawing.Point(56, 32);
            this.hostName.Name = "hostName";
            this.hostName.Size = new System.Drawing.Size(96, 20);
            this.hostName.TabIndex = 2;
            this.hostName.Text = "localhost";
            // 
            // label4
            // 
            this.label4.Location = new System.Drawing.Point(16, 64);
            this.label4.Name = "label4";
            this.label4.Size = new System.Drawing.Size(168, 23);
            this.label4.TabIndex = 5;
            this.label4.Text = "Number of connections to make:";
            // 
            // numConnections
            // 
            this.numConnections.Location = new System.Drawing.Point(200, 64);
            this.numConnections.Name = "numConnections";
            this.numConnections.Size = new System.Drawing.Size(56, 20);
            this.numConnections.TabIndex = 6;
            this.numConnections.Text = "100";
            // 
            // groupBox1
            // 
            this.groupBox1.Location = new System.Drawing.Point(8, 8);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(296, 152);
            this.groupBox1.TabIndex = 0;
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "Connections";
            // 
            // groupBox2
            // 
            this.groupBox2.Location = new System.Drawing.Point(8, 168);
            this.groupBox2.Name = "groupBox2";
            this.groupBox2.Size = new System.Drawing.Size(296, 120);
            this.groupBox2.TabIndex = 11;
            this.groupBox2.TabStop = false;
            this.groupBox2.Text = "Send/Receive";
            // 
            // groupBox3
            // 
            this.groupBox3.Location = new System.Drawing.Point(8, 296);
            this.groupBox3.Name = "groupBox3";
            this.groupBox3.Size = new System.Drawing.Size(296, 136);
            this.groupBox3.TabIndex = 19;
            this.groupBox3.TabStop = false;
            this.groupBox3.Text = "Most Recent Exception";
            // 
            // disconnect
            // 
            this.disconnect.Location = new System.Drawing.Point(144, 128);
            this.disconnect.Name = "disconnect";
            this.disconnect.TabIndex = 10;
            this.disconnect.Text = "Disconnect";
            this.disconnect.Click += new System.EventHandler(this.disconnect_Click);
            // 
            // connectionCount
            // 
            this.connectionCount.Location = new System.Drawing.Point(128, 96);
            this.connectionCount.Name = "connectionCount";
            this.connectionCount.Size = new System.Drawing.Size(56, 23);
            this.connectionCount.TabIndex = 8;
            this.connectionCount.Text = "0";
            // 
            // label2
            // 
            this.label2.Location = new System.Drawing.Point(16, 96);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(104, 23);
            this.label2.TabIndex = 7;
            this.label2.Text = "Active connections:";
            // 
            // connect
            // 
            this.connect.Location = new System.Drawing.Point(56, 128);
            this.connect.Name = "connect";
            this.connect.TabIndex = 9;
            this.connect.Text = "Connect";
            this.connect.Click += new System.EventHandler(this.connect_Click);
            // 
            // label6
            // 
            this.label6.Location = new System.Drawing.Point(16, 200);
            this.label6.Name = "label6";
            this.label6.Size = new System.Drawing.Size(72, 23);
            this.label6.TabIndex = 12;
            this.label6.Text = "Text to send:";
            // 
            // textToSend
            // 
            this.textToSend.Location = new System.Drawing.Point(96, 200);
            this.textToSend.MaxLength = 30;
            this.textToSend.Name = "textToSend";
            this.textToSend.Size = new System.Drawing.Size(120, 20);
            this.textToSend.TabIndex = 13;
            this.textToSend.Text = "";
            // 
            // sendText
            // 
            this.sendText.Location = new System.Drawing.Point(216, 200);
            this.sendText.Name = "sendText";
            this.sendText.TabIndex = 14;
            this.sendText.Text = "Send";
            this.sendText.Click += new System.EventHandler(this.sendText_Click);
            // 
            // label7
            // 
            this.label7.Location = new System.Drawing.Point(16, 232);
            this.label7.Name = "label7";
            this.label7.Size = new System.Drawing.Size(96, 23);
            this.label7.TabIndex = 15;
            this.label7.Text = "Most recent reply:";
            // 
            // replyText
            // 
            this.replyText.Location = new System.Drawing.Point(120, 232);
            this.replyText.Name = "replyText";
            this.replyText.Size = new System.Drawing.Size(168, 23);
            this.replyText.TabIndex = 16;
            // 
            // lastException
            // 
            this.lastException.Location = new System.Drawing.Point(16, 320);
            this.lastException.Multiline = true;
            this.lastException.Name = "lastException";
            this.lastException.ReadOnly = true;
            this.lastException.ScrollBars = System.Windows.Forms.ScrollBars.Both;
            this.lastException.Size = new System.Drawing.Size(280, 72);
            this.lastException.TabIndex = 20;
            this.lastException.Text = "";
            // 
            // clearException
            // 
            this.clearException.Location = new System.Drawing.Point(216, 400);
            this.clearException.Name = "clearException";
            this.clearException.TabIndex = 21;
            this.clearException.Text = "Clear";
            this.clearException.Click += new System.EventHandler(this.clearException_Click);
            // 
            // label5
            // 
            this.label5.Location = new System.Drawing.Point(16, 256);
            this.label5.Name = "label5";
            this.label5.Size = new System.Drawing.Size(120, 23);
            this.label5.TabIndex = 17;
            this.label5.Text = "Most recent endpoint:";
            // 
            // lastEndpoint
            // 
            this.lastEndpoint.Location = new System.Drawing.Point(136, 256);
            this.lastEndpoint.Name = "lastEndpoint";
            this.lastEndpoint.Size = new System.Drawing.Size(160, 23);
            this.lastEndpoint.TabIndex = 18;
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(312, 438);
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                          this.label5,
                                                                          this.lastException,
                                                                          this.clearException,
                                                                          this.label6,
                                                                          this.textToSend,
                                                                          this.sendText,
                                                                          this.label7,
                                                                          this.replyText,
                                                                          this.disconnect,
                                                                          this.connectionCount,
                                                                          this.label2,
                                                                          this.connect,
                                                                          this.portNum,
                                                                          this.label1,
                                                                          this.label3,
                                                                          this.hostName,
                                                                          this.label4,
                                                                          this.numConnections,
                                                                          this.groupBox3,
                                                                          this.groupBox1,
                                                                          this.lastEndpoint,
                                                                          this.groupBox2});
            this.Name = "Form1";
            this.Text = "Client";
            this.ResumeLayout(false);

        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

        private void connect_Click(object sender, System.EventArgs e)
        {
            int connectionsToMake = int.Parse(numConnections.Text);

            IPHostEntry hostInfo = Dns.Resolve(hostName.Text);

            if( hostInfo.AddressList.Length == 0 )
            {
                MessageBox.Show("Failed to resolve specified host name.");
                return;
            }

            IPAddress ipAddress = hostInfo.AddressList[0];
            IPEndPoint ipEndpoint = new IPEndPoint(ipAddress, int.Parse(portNum.Text));

            for( int n = 0; n < connectionsToMake; n++ )
            {
                Socket s = new Socket( AddressFamily.InterNetwork,
                                           SocketType.Stream,
                                           ProtocolType.Tcp );
                
                ServerInfo si = new ServerInfo(s);

                try
                {
                    s.BeginConnect(ipEndpoint, connectedProc, si);
                }
                catch( Exception eConn )
                {
                    lastExceptionUpdater.ExceptionText = eConn;
                }

                // Pause for a breath to we don't flood the server's
                // backlog queue.
                //
                Thread.Sleep(10);
            }
        }

        void OnConnect( IAsyncResult ar )
        {
            ServerInfo si = (ServerInfo)ar.AsyncState;

            try
            {
                si.socket.EndConnect(ar);

                if( si.socket.Connected )
                {
                    connectionList.Add(si);
                    UpdateConnectionCount(1);
                    RecordLastEndpoint(si);

                    try
                    {
                        si.socket.BeginReceive( si.replyBuf, 0, si.replyBuf.Length,
                                                SocketFlags.None, receiveProc, si );
                    }
                    catch( Exception e )
                    {
                        lastExceptionUpdater.ExceptionText = e;
                    }
                }
            }
            catch( Exception e )
            {
                lastExceptionUpdater.ExceptionText = e;
            }
        }

        void RecordLastEndpoint( ServerInfo si )
        {
            lastServer = si;
            IPEndPoint ipEndpoint = (IPEndPoint)si.socket.RemoteEndPoint;
            lastEndpointUpdater.Text =
                string.Format( "{0}, {1}",
                               Dns.GetHostByAddress(ipEndpoint.Address).HostName,
                               ipEndpoint.Address );
        }

        private void disconnect_Click(object sender, System.EventArgs e)
        {
            // Work on a temp list because each socket shutdown is going
            // to trigger a remove from the list.
            //
            ArrayList temp = (ArrayList)connectionList.Clone();

            foreach( ServerInfo si in temp )
            {
                si.shutdown = true;
                si.socket.Shutdown(SocketShutdown.Both);
                si.socket.Close();
            }
        }

        void OnReceive( IAsyncResult ar )
        {
            ServerInfo si = (ServerInfo)ar.AsyncState;

            try
            {
                if( si.shutdown )
                {
                    // The 'disconnect' button has been pressed, triggering
                    // our pending receive operations to fail.
                    //
                    DropConnection(si);
                    return;
                }

                // Since the client won't actually be sending data, the
                // following call to EndReceive should either throw an exception
                // or return 0.
                //
                int cbReceived = si.socket.EndReceive(ar);

                if( cbReceived > 0 )
                {
                    replyTextUpdater.Text = Encoding.ASCII.GetString(si.replyBuf, 0, cbReceived);

                    // On the off chance that we don't fail, issue a new
                    // async receive.
                    //
                    si.socket.BeginReceive( si.replyBuf, 0, si.replyBuf.Length,
                                            SocketFlags.None, receiveProc, si );
                }
                else
                {
                    DropConnection(si);
                }
            }
            catch( Exception e )
            {
                lastExceptionUpdater.ExceptionText = e;
                DropConnection(si);
            }
        }

        void DropConnection( ServerInfo si )
        {
            bool removed = false;

            lock( connectionList.SyncRoot )
            {
                try
                {
                    if( connectionList.Contains(si) )
                    {
                        connectionList.Remove(si);
                        removed = true;
                    }
                }
                catch( Exception e )
                {
                    lastExceptionUpdater.ExceptionText = e;
                }
            }

            if( removed )
            {
                UpdateConnectionCount(-1);
            }
        }

        void UpdateConnectionCount( int howMany )
        {
            if( connectionCount.InvokeRequired )
            {
                connectionCount.BeginInvoke(updateConnectionCountProc, new object[]{howMany});
                return;
            }

            connectionCount.Text = (int.Parse(connectionCount.Text) + howMany).ToString();
        }

        private void clearException_Click(object sender, System.EventArgs e)
        {
            lastException.Text = "";
        }

        private void sendText_Click(object sender, System.EventArgs e)
        {
            byte[] dataBuf = Encoding.ASCII.GetBytes(textToSend.Text);
            lastServer.socket.BeginSend(dataBuf, 0, dataBuf.Length, SocketFlags.None, null, null);
        }
    }
}
