// server.cs
//
// This program allows an arbitrary # of connections to be accepted from one or more instances
// of the associated client program in order to test the connection limits of a given system.
// It also demonstrates how to use completely asynchronous I/O operations on sockets for both
// connection management as well as I/O.
//
// Refer to client.cs for the client implementation and utils.cs for the shared
// ThreadSafeControlUpdater utility class.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace Server
{
    // ClientInfo
    //
    // An instance of this class is allocated for each client
    // connection accepted.
    //
    class ClientInfo
    {
        public ClientInfo( Socket s )
        {
            socket = s;
        }

        internal readonly Socket socket;
        internal byte[] dataBuf = new byte[32];
    }

	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
        delegate void UpdateClientCountProc( int howMany );

        private bool started = false;
        private bool reverseReply = false;
        private bool shuttingDown = false;
        private Socket acceptSocket = null;
        private AsyncCallback acceptProc;
        private AsyncCallback receiveProc;
        private ArrayList clientSockets = ArrayList.Synchronized(new ArrayList());

        // Thread-affine control updaters for controls being updated
        // from I/O threads.
        //
        private ThreadSafeControlUpdater lastExceptionUpdater;
        private ThreadSafeControlUpdater requestTextUpdater;
        private ThreadSafeControlUpdater replyTextUpdater;
        private UpdateClientCountProc updateConnectCountProc;

        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.TextBox portNum;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label clientCount;
        private System.Windows.Forms.TextBox lastException;
        private System.Windows.Forms.Button clearException;
        private System.Windows.Forms.GroupBox groupBox3;
        private System.Windows.Forms.Label label7;
        private System.Windows.Forms.Label replyText;
        private System.Windows.Forms.GroupBox groupBox2;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.GroupBox groupBox1;
        private System.Windows.Forms.Button startStop;
        private System.Windows.Forms.Label requestText;
        private System.Windows.Forms.CheckBox reverseReplyCheckBox;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

            acceptProc = new AsyncCallback(OnAccept);
            receiveProc = new AsyncCallback(OnReceive);

            requestTextUpdater = new ThreadSafeControlUpdater(requestText);
            replyTextUpdater = new ThreadSafeControlUpdater(replyText);
            lastExceptionUpdater = new ThreadSafeControlUpdater(lastException);
            updateConnectCountProc = new UpdateClientCountProc(UpdateClientCount);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.label1 = new System.Windows.Forms.Label();
            this.portNum = new System.Windows.Forms.TextBox();
            this.label2 = new System.Windows.Forms.Label();
            this.clientCount = new System.Windows.Forms.Label();
            this.lastException = new System.Windows.Forms.TextBox();
            this.clearException = new System.Windows.Forms.Button();
            this.groupBox3 = new System.Windows.Forms.GroupBox();
            this.label7 = new System.Windows.Forms.Label();
            this.replyText = new System.Windows.Forms.Label();
            this.groupBox2 = new System.Windows.Forms.GroupBox();
            this.label3 = new System.Windows.Forms.Label();
            this.requestText = new System.Windows.Forms.Label();
            this.reverseReplyCheckBox = new System.Windows.Forms.CheckBox();
            this.groupBox1 = new System.Windows.Forms.GroupBox();
            this.startStop = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // label1
            // 
            this.label1.Location = new System.Drawing.Point(16, 32);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(40, 23);
            this.label1.TabIndex = 1;
            this.label1.Text = "Port:";
            // 
            // portNum
            // 
            this.portNum.Location = new System.Drawing.Point(64, 32);
            this.portNum.Name = "portNum";
            this.portNum.Size = new System.Drawing.Size(56, 20);
            this.portNum.TabIndex = 2;
            this.portNum.Text = "999";
            // 
            // label2
            // 
            this.label2.Location = new System.Drawing.Point(136, 32);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(104, 23);
            this.label2.TabIndex = 3;
            this.label2.Text = "Active connections:";
            // 
            // clientCount
            // 
            this.clientCount.Location = new System.Drawing.Point(248, 32);
            this.clientCount.Name = "clientCount";
            this.clientCount.Size = new System.Drawing.Size(48, 23);
            this.clientCount.TabIndex = 4;
            this.clientCount.Text = "0";
            // 
            // lastException
            // 
            this.lastException.Location = new System.Drawing.Point(16, 232);
            this.lastException.Multiline = true;
            this.lastException.Name = "lastException";
            this.lastException.ReadOnly = true;
            this.lastException.ScrollBars = System.Windows.Forms.ScrollBars.Both;
            this.lastException.Size = new System.Drawing.Size(280, 72);
            this.lastException.TabIndex = 13;
            this.lastException.Text = "";
            // 
            // clearException
            // 
            this.clearException.Location = new System.Drawing.Point(216, 312);
            this.clearException.Name = "clearException";
            this.clearException.TabIndex = 14;
            this.clearException.Text = "Clear";
            this.clearException.Click += new System.EventHandler(this.clearException_Click);
            // 
            // groupBox3
            // 
            this.groupBox3.Location = new System.Drawing.Point(8, 208);
            this.groupBox3.Name = "groupBox3";
            this.groupBox3.Size = new System.Drawing.Size(296, 136);
            this.groupBox3.TabIndex = 12;
            this.groupBox3.TabStop = false;
            this.groupBox3.Text = "Most Recent Exception";
            // 
            // label7
            // 
            this.label7.Location = new System.Drawing.Point(16, 168);
            this.label7.Name = "label7";
            this.label7.Size = new System.Drawing.Size(80, 23);
            this.label7.TabIndex = 10;
            this.label7.Text = "Last reply:";
            // 
            // replyText
            // 
            this.replyText.Location = new System.Drawing.Point(104, 168);
            this.replyText.Name = "replyText";
            this.replyText.Size = new System.Drawing.Size(184, 23);
            this.replyText.TabIndex = 11;
            // 
            // groupBox2
            // 
            this.groupBox2.Location = new System.Drawing.Point(8, 96);
            this.groupBox2.Name = "groupBox2";
            this.groupBox2.Size = new System.Drawing.Size(296, 104);
            this.groupBox2.TabIndex = 6;
            this.groupBox2.TabStop = false;
            this.groupBox2.Text = "Send/Receive";
            // 
            // label3
            // 
            this.label3.Location = new System.Drawing.Point(16, 144);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(80, 23);
            this.label3.TabIndex = 8;
            this.label3.Text = "Last request:";
            // 
            // requestText
            // 
            this.requestText.Location = new System.Drawing.Point(104, 144);
            this.requestText.Name = "requestText";
            this.requestText.Size = new System.Drawing.Size(184, 23);
            this.requestText.TabIndex = 9;
            // 
            // reverseReplyCheckBox
            // 
            this.reverseReplyCheckBox.Location = new System.Drawing.Point(72, 112);
            this.reverseReplyCheckBox.Name = "reverseReplyCheckBox";
            this.reverseReplyCheckBox.Size = new System.Drawing.Size(160, 24);
            this.reverseReplyCheckBox.TabIndex = 7;
            this.reverseReplyCheckBox.Text = "Reverse text when replying";
            this.reverseReplyCheckBox.Click += new System.EventHandler(this.reverseReplyCheckBox_Click);
            // 
            // groupBox1
            // 
            this.groupBox1.Location = new System.Drawing.Point(8, 8);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(296, 88);
            this.groupBox1.TabIndex = 0;
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "Connections";
            // 
            // startStop
            // 
            this.startStop.Location = new System.Drawing.Point(119, 64);
            this.startStop.Name = "startStop";
            this.startStop.TabIndex = 5;
            this.startStop.Text = "Start";
            this.startStop.Click += new System.EventHandler(this.startStop_Click);
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(312, 350);
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                          this.startStop,
                                                                          this.reverseReplyCheckBox,
                                                                          this.label3,
                                                                          this.requestText,
                                                                          this.label7,
                                                                          this.replyText,
                                                                          this.groupBox2,
                                                                          this.lastException,
                                                                          this.clearException,
                                                                          this.groupBox3,
                                                                          this.portNum,
                                                                          this.label1,
                                                                          this.label2,
                                                                          this.clientCount,
                                                                          this.groupBox1});
            this.Name = "Form1";
            this.Text = "Server";
            this.ResumeLayout(false);

        }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

        private void startStop_Click(object sender, System.EventArgs e)
        {
            if( !started )
            {
                clientCount.Text = "0";

                // Start things up...
                //
                IPEndPoint ipEndpoint = new IPEndPoint(IPAddress.Any, int.Parse(portNum.Text));

                shuttingDown = false;
                acceptSocket =
                    new Socket( AddressFamily.InterNetwork, SocketType.Stream,
                                ProtocolType.Tcp );
                acceptSocket.Bind(ipEndpoint);
                acceptSocket.Listen(10);
                acceptSocket.BeginAccept(acceptProc, null);
            }
            else
            {
                // Shut things down (work on a clone of clientSockets
                // because each socket shutdown is going to trigger
                // a call to OnReceive, which will remove the client
                // from the connection list).
                //
                ArrayList temp = (ArrayList)clientSockets.Clone();

                foreach( ClientInfo ci in temp )
                {
                    DropClient(ci.socket);
                }

                // This will trigger OnAccept.
                //
                shuttingDown = true;
                acceptSocket.Close();
            }

            started = !started;

            if( started )
            {
                startStop.Text = "Stop";
                portNum.Enabled = false;
            }
            else
            {
                startStop.Text = "Start";
                portNum.Enabled = true;
            }
        }

        void OnAccept( IAsyncResult ar )
        {
            try
            {
                if( shuttingDown )
                {
                    // The "stop" button was pressed, which caused the app to close
                    // this socket and reset itself to be started again.
                    return;
                }

                Socket clientSocket = acceptSocket.EndAccept(ar);

                if( !clientSocket.Connected )
                    return;

                ClientInfo ci = new ClientInfo(clientSocket);

                // Register the new client.
                //
                clientSockets.Add(ci);

                // Issue an async read just so that we can detect
                // a disconnect w/o polling.
                try
                {
                    clientSocket.BeginReceive( ci.dataBuf, 0, ci.dataBuf.Length,
                                               SocketFlags.None, receiveProc, ci);

                    // Update the UI.
                    //
                    UpdateClientCount(1);
                }
                catch( Exception e )
                {
                    lastExceptionUpdater.ExceptionText = e;
                }

                // Get ready for next incoming connection.
                //
                acceptSocket.BeginAccept(acceptProc, null);
            }
            catch( Exception e )
            {
                lastExceptionUpdater.ExceptionText = e;
            }
        }
        
        void OnReceive( IAsyncResult ar )
        {
            ClientInfo ci = (ClientInfo)ar.AsyncState;

            try
            {
                // Since the client won't actually be sending data, the
                // following call to EndReceive should either throw an exception
                // or return 0.
                //
                int cbReceived = ci.socket.EndReceive(ar);

                if( cbReceived > 0 )
                {
                    // If we didn't fail, echo the data back and
                    // then queue up another receive buffer.
                    //
                    try
                    {
                        // Extract string, possibly reversing it.
                        //
                        string replyMsg = Encoding.ASCII.GetString(ci.dataBuf, 0, cbReceived);
                        char[] replyChars = replyMsg.ToCharArray();

                        if( reverseReply )
                        {
                            Array.Reverse(replyChars);
                        }

                        // Send reply.
                        //
                        byte[] replyBuf = Encoding.ASCII.GetBytes(replyChars);
                        ci.socket.BeginSend(replyBuf, 0, replyBuf.Length, SocketFlags.None, null, null);

                        // Queue up for another request.
                        //
                        ci.socket.BeginReceive( ci.dataBuf, 0, ci.dataBuf.Length,
                                                SocketFlags.None, receiveProc, ci);

                        requestTextUpdater.Text = replyMsg;
                        replyTextUpdater.Text = Encoding.ASCII.GetString(replyBuf);
                    }
                    catch( Exception e )
                    {
                        DropClient(ci.socket);
                        lastExceptionUpdater.ExceptionText = e;
                    }
                }
                else
                {
                    DropClient(ci.socket);
                }
            }
            catch( Exception e )
            {
                DropClient(ci.socket);
                lastExceptionUpdater.ExceptionText = e;
            }
        }

        void DropClient( Socket s )
        {
            try
            {
                s.Shutdown(SocketShutdown.Both);
            }
            catch
            {
            }

            bool removed = false;

            lock( clientSockets.SyncRoot )
            {
                try
                {
                    // Find and remove the associated ClientInfo object from
                    // our list.
                    //
                    for( int n = 0; !removed && (n < clientSockets.Count); n++ )
                    {
                        ClientInfo ci = (ClientInfo)clientSockets[n];

                        if( ci.socket == s )
                        {
                            clientSockets.Remove(ci);
                            removed = true;
                        }
                    }
                }
                catch( Exception e )
                {
                    lastExceptionUpdater.ExceptionText = e;
                }
            }

            if( removed )
            {
                UpdateClientCount(-1);
            }
        }

        void UpdateClientCount( int howMany )
        {
            if( clientCount.InvokeRequired )
            {
                clientCount.BeginInvoke(updateConnectCountProc, new object[]{howMany});
                return;
            }

            clientCount.Text = (int.Parse(clientCount.Text) + howMany).ToString();
        }

        private void clearException_Click(object sender, System.EventArgs e)
        {
            lastException.Text = "";
        }

        private void reverseReplyCheckBox_Click(object sender, System.EventArgs e)
        {
            reverseReply = reverseReplyCheckBox.Checked;
        }
	}
}
