// utils.cs
//
// This file contains the ThreadSafeControlUpdater utility class that facilitates
// the common chore of updating a control's Text property in a multithreaded
// environment such as this one.  The Text property of this class provides that
// generic functionality.  The ExceptionText property is more specific to this
// particular program, but uses the same technique for updating the Text
// property of a given control based on the Exception text and stack trace
// of a specified exception.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Windows.Forms;

public class ThreadSafeControlUpdater
{
    delegate void SetTextProc( string newText );

    Control targetControl;
    SetTextProc setText;

    public ThreadSafeControlUpdater( Control ctrl )
    {
        targetControl = ctrl;
        setText = new SetTextProc(SetText);
    }

    public string Text
    {
        set { SetText(value); }
    }

    public Exception ExceptionText
    {
        set { SetText(string.Format("{0}\r\n{1}", value.Message, value.StackTrace)); }
    }

    void SetText( string newText )
    {
        if( targetControl.InvokeRequired )
        {
            targetControl.BeginInvoke(setText, new object[]{newText});
            return;
        }

        targetControl.Text = newText;
    }
}
